(function(_window, _document, _log, _utils) {
    'use strict';

    var retryDomains = [
            'yahoo.com',
            'ymail.com'
        ],
        regexDomainPattern = '^https?\:\/\/[^\/|\?|\:]*\.';

    function retryCallback(ev) {
        var imgEl = ev.currentTarget,
            url = imgEl.getAttribute('src'),
            shouldRetry;

        shouldRetry = _utils.findFirst( retryDomains, function(domain) {
            var regex = new RegExp(regexDomainPattern + domain, 'i');

            return regex.test(url);
        });

        if (shouldRetry) {
            _log.d('retrying url: ' + url);

            // force reload
            imgEl.setAttribute('src', url);
        }

        // remove listener. only allow single retry
        removeAllListeners(ev);
    }

    function removeAllListeners(ev) {
        var imgEl = ev.currentTarget;

        imgEl.removeEventListener('error', retryCallback);
        imgEl.removeEventListener('load', removeAllListeners);
    }
    /**
     * Find all img tags and add any necessary callbacks and/or attributes.
     */
    function process() {
        var imgs = _document.getElementsByTagName('img'),
            i;

        for(i=0; i < imgs.length; i++) {
            imgs[i].addEventListener('error', retryCallback);
            imgs[i].addEventListener('load', removeAllListeners);
        }
    }

    /**
     * Binds event handler to retry img fetch once on failure.
     *
     * NOTE: Call in head of document (before DOMContentLoaded event).
     */
    function bindListeners() {
        _document.addEventListener('DOMContentLoaded', process.bind(this));
    }

    _window.imgElements = {
        bindListeners: bindListeners
    };
})(window, document, window.log, window.utils);
