(function(_window) {
    'use strict';

    var utils = {
        /**
         * Return true if the value is an Integer.
         *
         * Uses ES6 function if defined.
         *
         * @param {*} val
         */
        isInteger: Number.isInteger || function(val) {
            return typeof val === "number" &&
                   isFinite(val) &&
                   Math.floor(val) === val;
        },

        /**
         * Functional utility to loop through an array calling a
         * callback each time to test for a condition to be meet, and
         * exit on first find.
         *
         * @param {Array} arr
         * @param {Function} cb
         * @returns {Boolean}
         */
        findFirst: function(arr, cb) {
            var i;

            for(i=0; i < arr.length; i++) {
                if (cb(arr[i])) {
                    return true;
                }
            }

            return false;
        },

        // Port from underscore
        debounce: function(func, wait, immediate) {
            var timeout;
            return function() {
                var context = this, args = arguments;
                var later = function() {
                    timeout = null;
                    if (!immediate) {
                        func.apply(context, args);
                    }
                };
                var callNow = immediate && !timeout;
                clearTimeout(timeout);
                timeout = setTimeout(later, wait);
                if (callNow) {
                    func.apply(context, args);
                }
            };
        }
    };

    _window.utils = utils;
})(window);
