/*
 * Copyright (C) 2014 Yahoo Inc.
 */

/**
 * This file contains the global initialization functions which are called by the native Java code
 */

// Tags which are used to distinguish between the different recipient fields - To/CC/BCC
// They are initialized by the native code via init(). The values present here are the default values.
var TAG_TO_RECIPIENT = 11;
var TAG_CC_RECIPIENT = 12;
var TAG_BCC_RECIPIENT = 13;
var INVALID_CURSOR_OFFSET = -1;

// Constants which store element IDs
var COMPOSE_HEADER_ELEMENT_ID = "ymail_composeHeader";
var COMPOSE_CONTENT_ELEMENT_ID = "ymail_composeContent";

var TO_ROW_ELEMENT_ID = "ymail_toRow";
var CC_ROW_ELEMENT_ID = "ymail_ccRow";
var BCC_ROW_ELEMENT_ID = "ymail_bccRow";
var FROM_ROW_ELEMENT_ID = "ymail_fromRow";
var CC_BCC_FROM_SUMMARY_ROW_ELEMENT_ID = "ymail_ccBccFromSummaryRow";
var SUBJECT_ROW_ELEMENT_ID = "ymail_subjectRow";
var ATTACHMENTS_ROW_ELEMENT_ID = "ymail_attachmentsRow";
var ATTACHMENTS_LABEL_ELEMENT_ID = "ymail_attachmentsLabel";

var CC_BCC_FROM_SUMMARY_ROW_TEXT_ELEMENT_ID = "ymail_ccBccFromSummaryRowText";

var SELECTED_FROM_ADDRESS_ELEMENT_ID = "ymail_selectedFromAddress";

var TO_LOZENGE_CONTAINER_ELEMENT_ID = "ymail_toLozengeContainer";
var CC_LOZENGE_CONTAINER_ELEMENT_ID = "ymail_ccLozengeContainer";
var BCC_LOZENGE_CONTAINER_ELEMENT_ID = "ymail_bccLozengeContainer";

var TO_INPUT_ELEMENT_ID = "ymail_toInput";
var CC_INPUT_ELEMENT_ID = "ymail_ccInput";
var BCC_INPUT_ELEMENT_ID = "ymail_bccInput";

var SEARCH_SUGGESTION_ELEMENT_ID = "ymail_contactSearchSuggestionsContainer";

var TO_LOZENGE_SUMMARY_ELEMENT_ID = "ymail_toLozengeSummary";
var CC_LOZENGE_SUMMARY_ELEMENT_ID = "ymail_ccLozengeSummary";
var BCC_LOZENGE_SUMMARY_ELEMENT_ID = "ymail_bccLozengeSummary";

var SUBJECT_INPUT_ELEMENT_ID = "ymail_subjectField";

var ATTACHMENTS_ELEMENT_ID = "ymail_attachments";

/**
 * Called by native code to initialize the JavaScript
 */
function init(invalidCursorOffset, toTag, ccTag, bccTag, numFromAddresses, toString, ccString, bccString, subjectString, fromString, accessibilityComposeContent, inlineAttachmentNoCacheQueryParam) {
    // Set recipient field tag values
    TAG_TO_RECIPIENT = toTag;
    TAG_CC_RECIPIENT = ccTag;
    TAG_BCC_RECIPIENT = bccTag;

    // Set cursor position
    INVALID_CURSOR_OFFSET = invalidCursorOffset;

    // Store node references
    var headerContainerElement = document.getElementById(COMPOSE_HEADER_ELEMENT_ID);
    var composeContentElement =  document.getElementById(COMPOSE_CONTENT_ELEMENT_ID);

    var toRowElement = document.getElementById(TO_ROW_ELEMENT_ID);
    var ccRowElement = document.getElementById(CC_ROW_ELEMENT_ID);
    var bccRowElement = document.getElementById(BCC_ROW_ELEMENT_ID);
    var fromRowElement = document.getElementById(FROM_ROW_ELEMENT_ID);
    var ccBccFromSummaryRowElement = document.getElementById(CC_BCC_FROM_SUMMARY_ROW_ELEMENT_ID);
    var subjectRowElement = document.getElementById(SUBJECT_ROW_ELEMENT_ID);
    var attachmentsRowElement = document.getElementById(ATTACHMENTS_ROW_ELEMENT_ID);

    var ccBccFromSummaryRowTextElement = document.getElementById(CC_BCC_FROM_SUMMARY_ROW_TEXT_ELEMENT_ID);
    var fromAddressTextElement = document.getElementById(SELECTED_FROM_ADDRESS_ELEMENT_ID);

    var toLozengeContainerElement = document.getElementById(TO_LOZENGE_CONTAINER_ELEMENT_ID);
    var ccLozengeContainerElement = document.getElementById(CC_LOZENGE_CONTAINER_ELEMENT_ID);
    var bccLozengeContainerElement = document.getElementById(BCC_LOZENGE_CONTAINER_ELEMENT_ID);

    var toInputElement = document.getElementById(TO_INPUT_ELEMENT_ID);
    var ccInputElement = document.getElementById(CC_INPUT_ELEMENT_ID);
    var bccInputElement = document.getElementById(BCC_INPUT_ELEMENT_ID);

    var searchSuggestionElement = document.getElementById(SEARCH_SUGGESTION_ELEMENT_ID);

    var toLozengeSummaryElement = document.getElementById(TO_LOZENGE_SUMMARY_ELEMENT_ID);
    var ccLozengeSummaryElement = document.getElementById(CC_LOZENGE_SUMMARY_ELEMENT_ID);
    var bccLozengeSummaryElement = document.getElementById(BCC_LOZENGE_SUMMARY_ELEMENT_ID);

    var subjectInputElement = document.getElementById(SUBJECT_INPUT_ELEMENT_ID);

    var attachmentsLabelElement = document.getElementById(ATTACHMENTS_LABEL_ELEMENT_ID);

    var attachmentsElement = document.getElementById(ATTACHMENTS_ELEMENT_ID);

    // Initialize text with correct translations
    document.getElementById("ymail_toLabel").textContent = toString;
    document.getElementById("ymail_ccLabel").textContent = ccString;
    document.getElementById("ymail_bccLabel").textContent = bccString;
    subjectInputElement.setAttribute("placeholder", subjectString);
    document.getElementById("ymail_fromLabel").textContent = fromString;

    // Start lozenges controller
    lozengeController.create(headerContainerElement, composeContentElement, toRowElement, toLozengeContainerElement, toLozengeSummaryElement, toInputElement, TAG_TO_RECIPIENT);
    lozengeController.create(headerContainerElement, composeContentElement, ccRowElement, ccLozengeContainerElement, ccLozengeSummaryElement, ccInputElement, TAG_CC_RECIPIENT);
    lozengeController.create(headerContainerElement, composeContentElement, bccRowElement, bccLozengeContainerElement, bccLozengeSummaryElement, bccInputElement, TAG_BCC_RECIPIENT);

    // Start the type ahead contact search controller
    contactSearchController.create(toInputElement, searchSuggestionElement, toRowElement, [ ccRowElement, bccRowElement, fromRowElement, ccBccFromSummaryRowElement, subjectRowElement, attachmentsRowElement, composeContentElement], TAG_TO_RECIPIENT);
    contactSearchController.create(ccInputElement, searchSuggestionElement, ccRowElement, [ toRowElement, bccRowElement, fromRowElement, ccBccFromSummaryRowElement, subjectRowElement, attachmentsRowElement, composeContentElement], TAG_CC_RECIPIENT);
    contactSearchController.create(bccInputElement, searchSuggestionElement, bccRowElement, [ toRowElement, ccRowElement, fromRowElement, ccBccFromSummaryRowElement, subjectRowElement, attachmentsRowElement, composeContentElement], TAG_BCC_RECIPIENT);

    // Start CC/BCC/From summary control logic
    ccBccFromSummaryFieldController.init(ccBccFromSummaryRowElement, ccBccFromSummaryRowTextElement, headerContainerElement, ccRowElement, ccInputElement, bccInputElement, bccRowElement, fromRowElement, fromAddressTextElement, ccLozengeContainerElement, bccLozengeContainerElement, CC_INPUT_ELEMENT_ID, BCC_INPUT_ELEMENT_ID);

    // Subject watcher
    subjectTextController.init(subjectRowElement, subjectInputElement);

    // Attachments controller
    attachmentsBandController.init(attachmentsRowElement, attachmentsElement, attachmentsLabelElement, composeContentElement, inlineAttachmentNoCacheQueryParam);

    // Start the global focus tracker, so that we can restore the focus correct on configuration change
    document.body.addEventListener("focus", globalFocusTracker, true);

    // Start the auto save controller
    autoSaveController.init(composeContentElement);

    // Start the compose content focus controller
    composeContentFocusController.init(composeContentElement);

    // Start the tap to delete image controller
    imageTapDeleteController.init(composeContentElement);

    // Set accessibility strings
    subjectInputElement.setAttribute("aria-label", subjectString);
    composeContentElement.setAttribute("aria-label", accessibilityComposeContent);

    // Unfortunately, selection is broken prior to KitKat due to webviewcoreglue bugs.
    // So, we can only extract links on WebView's using Chrome.
    if (yMailBridge.isKitKatOrAbove()) {
        // listen for URLs being inserted by the user
        LinkExtractor.watch(composeContentElement);
    }
}

/**
 * Tracks the currently focused element and saves the id of the element through native code.
 * Used to restore the focus after configuration changes such as device rotations.
 *
 * @param event the JavaScript FocusEvent object
 */
function globalFocusTracker(event) {
    yMailBridge.setCurrentlyFocusedElementId(event.target.id);
    yMailBridge.setCurrentlyFocusedBodyNodeCursorOffset(INVALID_CURSOR_OFFSET);
}

/**
 * A global util function which is used by various components to pre-process and save the content of the message being composed
 * back to native code.
 *
 * @param contentNode the DOM Node which has the composed message's content
 * @param shouldPersist a boolean which indicates if the composed message's content should also be saved or not
 */
function syncComposeContent(composeContentNode, shouldPersist) {
    yMailBridge.setContent(composeContentNode.innerHTML, shouldPersist);
}
