/*
 * Copyright (C) 2015 Yahoo Inc.
 */

/**
 * A placeholder id prefix which is attached to every image node which doesn't have an id.
 */
var IMAGE_TAP_CONTROLLER_PLACEHOLDER_ID_PREFIX = "ymail_image_tap_delete_controller_";

/**
 * An object which handles clicks on images and throws a confirmation dialog asking the user if he wants to delete it or not.
 */
var imageTapDeleteController = {
    composeContentNode: null,

    /**
     * A method which starts the imageTapDeleteController
     */
    init: function(composeContentNode) {
        this.composeContentNode = composeContentNode;
        this.composeContentNode.addEventListener("click", this.handleComposeContentClick.bind(this), true);
    },

    /**
     * Handles clicks on the compose content.
     * If the click originated from an image node, invokes Java to throw a delete confirmation prompt to the user.
     */
    handleComposeContentClick: function(event) {
        if (event.srcElement && event.srcElement.tagName && event.srcElement.tagName.indexOf("IMG") >= 0) {
            var imageNode = event.srcElement;
            if (!imageNode.hasAttribute("id")) {
                imageNode.setAttribute("id", IMAGE_TAP_CONTROLLER_PLACEHOLDER_ID_PREFIX + Math.random());
                syncComposeContent(this.composeContentNode, false);
            }

            yMailBridge.showImageOptions(imageNode.id, imageNode.id.replace(ATTACHMENT_ID_PREFIX, ""));
        }
    },

    /**
     * Called back by Java to confirm that the user chose to delete the image
     */
    confirmDelete: function(imageNodeId) {
        var imageNode = document.getElementById(imageNodeId);
        if (imageNode) {
            imageNode.parentNode.removeChild(imageNode);
            syncComposeContent(this.composeContentNode, true);

            // Make sure the focus is on the compose content
            setTimeout(function() { this.composeContentNode.focus(); }.bind(this), 3);
        }
    }
};
